@@ -18,21 +18,22 @@
 from __future__ import annotations
 
 from contextlib import closing
-from typing import Any, Callable, Iterable, Mapping
+from typing import Any, Callable, Iterable, Mapping, Sequence
 
 import pandas as pd
 import pyexasol
-from pyexasol import ExaConnection
+from pyexasol import ExaConnection, ExaStatement
 
 from airflow.providers.common.sql.hooks.sql import DbApiHook, return_single_query_results
 
 
 class ExasolHook(DbApiHook):
-    """
-    Interact with Exasol.
+    """Interact with Exasol.
+
     You can specify the pyexasol ``compression``, ``encryption``, ``json_lib``
     and ``client_name``  parameters in the extra field of your connection
     as ``{"compression": True, "json_lib": "rapidjson", etc}``.
+
     See `pyexasol reference
     <https://github.com/badoo/pyexasol/blob/master/docs/REFERENCE.md#connect>`_
     for more details.
@@ -66,13 +67,14 @@ class ExasolHook(DbApiHook):
         return conn
 
     def get_pandas_df(self, sql: str, parameters: dict | None = None, **kwargs) -> pd.DataFrame:
-        """
-        Executes the sql and returns a pandas dataframe
+        """Execute the SQL and return a Pandas dataframe.
 
-        :param sql: the sql statement to be executed (str) or a list of
-            sql statements to execute
+        :param sql: The sql statement to be executed (str) or a list of
+            sql statements to execute.
         :param parameters: The parameters to render the SQL query with.
-        :param kwargs: (optional) passed into pyexasol.ExaConnection.export_to_pandas method
+
+        Other keyword arguments are all forwarded into
+        ``pyexasol.ExaConnection.export_to_pandas``.
         """
         with closing(self.get_conn()) as conn:
             df = conn.export_to_pandas(sql, query_params=parameters, **kwargs)
@@ -83,8 +85,7 @@ class ExasolHook(DbApiHook):
         sql: str | list[str],
         parameters: Iterable | Mapping | None = None,
     ) -> list[dict | tuple[Any, ...]]:
-        """
-        Executes the sql and returns a set of records.
+        """Execute the SQL and return a set of records.
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
@@ -95,8 +96,7 @@ class ExasolHook(DbApiHook):
                 return cur.fetchall()
 
     def get_first(self, sql: str | list[str], parameters: Iterable | Mapping | None = None) -> Any:
-        """
-        Executes the sql and returns the first resulting row.
+        """Execute the SQL and return the first resulting row.
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
@@ -113,8 +113,7 @@ class ExasolHook(DbApiHook):
         query_params: dict | None = None,
         export_params: dict | None = None,
     ) -> None:
-        """
-        Exports data to a file.
+        """Export data to a file.
 
         :param filename: Path to the file to which the data has to be exported
         :param query_or_table: the sql statement to be executed or table name to export
@@ -133,6 +132,31 @@ class ExasolHook(DbApiHook):
             )
         self.log.info("Data saved to %s", filename)
 
+    @staticmethod
+    def get_description(statement: ExaStatement) -> Sequence[Sequence]:
+        """Copied implementation from DB2-API wrapper.
+
+        For more info, see
+        https://github.com/exasol/pyexasol/blob/master/docs/DBAPI_COMPAT.md#db-api-20-wrapper
+
+        :param statement: Exasol statement
+        :return: description sequence of t
+        """
+        cols = []
+        for k, v in statement.columns().items():
+            cols.append(
+                (
+                    k,
+                    v.get("type", None),
+                    v.get("size", None),
+                    v.get("size", None),
+                    v.get("precision", None),
+                    v.get("scale", None),
+                    True,
+                )
+            )
+        return cols
+
     def run(
         self,
         sql: str | Iterable[str],
@@ -142,10 +166,10 @@ class ExasolHook(DbApiHook):
         split_statements: bool = False,
         return_last: bool = True,
     ) -> Any | list[Any] | None:
-        """
-        Runs a command or a list of commands. Pass a list of sql
-        statements to the sql parameter to get them to execute
-        sequentially
+        """Run a command or a list of commands.
+
+        Pass a list of SQL statements to the SQL parameter to get them to
+        execute sequentially.
 
         :param sql: the sql statement to be executed (str) or a list of
             sql statements to execute
@@ -176,18 +200,17 @@ class ExasolHook(DbApiHook):
             self.set_autocommit(conn, autocommit)
             results = []
             for sql_statement in sql_list:
-                with closing(conn.execute(sql_statement, parameters)) as cur:
+                with closing(conn.execute(sql_statement, parameters)) as exa_statement:
                     self.log.info("Running statement: %s, parameters: %s", sql_statement, parameters)
                     if handler is not None:
-                        result = handler(cur)
+                        result = handler(exa_statement)
                         if return_single_query_results(sql, return_last, split_statements):
                             _last_result = result
-                            _last_description = cur.description
+                            _last_columns = self.get_description(exa_statement)
                         else:
                             results.append(result)
-                            self.descriptions.append(cur.description)
-
-                    self.log.info("Rows affected: %s", cur.rowcount)
+                            self.descriptions.append(self.get_description(exa_statement))
+                    self.log.info("Rows affected: %s", exa_statement.rowcount)
 
             # If autocommit was set to False or db does not support autocommit, we do a manual commit.
             if not self.get_autocommit(conn):
@@ -196,14 +219,13 @@ class ExasolHook(DbApiHook):
         if handler is None:
             return None
         if return_single_query_results(sql, return_last, split_statements):
-            self.descriptions = [_last_description]
+            self.descriptions = [_last_columns]
             return _last_result
         else:
             return results
 
     def set_autocommit(self, conn, autocommit: bool) -> None:
-        """
-        Sets the autocommit flag on the connection
+        """Set the autocommit flag on the connection.
 
         :param conn: Connection to set autocommit setting to.
         :param autocommit: The autocommit setting to set.
@@ -216,14 +238,12 @@ class ExasolHook(DbApiHook):
         conn.set_autocommit(autocommit)
 
     def get_autocommit(self, conn) -> bool:
-        """
-        Get autocommit setting for the provided connection.
-        Return True if autocommit is set.
-        Return False if autocommit is not set or set to False or conn
-        does not support autocommit.
+        """Get autocommit setting for the provided connection.
 
         :param conn: Connection to get autocommit setting from.
-        :return: connection autocommit setting.
+        :return: connection autocommit setting. True if ``autocommit`` is set
+            to True on the connection. False if it is either not set, set to
+            False, or the connection does not support auto-commit.
         """
         autocommit = conn.attr.get("autocommit")
         if autocommit is None:
@@ -232,8 +252,9 @@ class ExasolHook(DbApiHook):
 
     @staticmethod
     def _serialize_cell(cell, conn=None) -> Any:
-        """
-        Exasol will adapt all arguments to the execute() method internally,
+        """Override to disable cell serialization.
+
+        Exasol will adapt all arguments to the ``execute()`` method internally,
         hence we return cell without any conversion.
 
         :param cell: The cell to insert into the table
@@ -241,3 +262,15 @@ class ExasolHook(DbApiHook):
         :return: The cell
         """
         return cell
+
+
+def exasol_fetch_all_handler(statement: ExaStatement) -> list[tuple] | None:
+    if statement.result_type == "resultSet":
+        return statement.fetchall()
+    return None
+
+
+def exasol_fetch_one_handler(statement: ExaStatement) -> list[tuple] | None:
+    if statement.result_type == "resultSet":
+        return statement.fetchone()
+    return None
